// Qt
#include "flowlayout.h"
#include <qlabel.h>
#include <qfile.h>
#include <qtimer.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qtextedit.h>
// KDE
#include <kmessagebox.h>
#include <kaboutapplication.h>
#include <kpopupmenu.h>
#include <kstdaction.h>
#include <kaction.h>
#include <kactioncollection.h>
#include <kactionclasses.h>
#include <kapplication.h>
#include <kdialogbase.h>
#include <kcolorbutton.h>
#include <kfontrequester.h>
//#include <kconfig.h>
//#include <kglobal.h>
#include <kiconloader.h>
//#include <klocale.h>

//#include <kdebug.h>

// CPUInfo
#include "cpuinfo.h"
#include "prefs.h"

extern "C" {
	KPanelApplet* init( QWidget *parent, const QString& configFile) {
		KGlobal::locale()->insertCatalogue("cpuinfo");
		return new CPUInfo(configFile, KPanelApplet::Normal,
		                      //KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
		                      KPanelApplet::About | KPanelApplet::Preferences,
		                      parent, "cpuinfo");
	}
}

CPUInfo::CPUInfo(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
		: KPanelApplet(configFile, type, actions, parent, name),
		QToolTip( this ),
		cachedWFH(0),
		cachedHeight(0),
		sampleRate(500),
		thermalItem(-1),
		frequencyItem(-1){
	// provide useful default values
	// these values are overwritten when reading the config
	if(cpudata.getThermalSources().size() > 0)
		thermalItem = 0;
	if(cpudata.getFrequencySources().size() > 0)
		frequencyItem = 0;

	aboutDialog = 0; // aboutData is created in the about() method
	prefsDlg = 0; // the prefs dialog is created in the preferences() method
	prefs = 0; // the prefs ui is created in the preferences() method

	// Get the current application configuration handle
	ksConfig = config();
	ksConfig->setGroup( "CPUInfo" );

	// create the timer
	refreshTimer = new QTimer( this, "refresh handler" );

	// create layout
	layout = new FlowLayout(this, orientation());
	layout->setSpacing(8);

	// create the labels
	cpuTempLabel = new QLabel(this);
	cpuFreqLabel = new QLabel(this);

	// enable transparency
//	setBackgroundMode(X11ParentRelative);
	cpuTempLabel->setBackgroundMode(X11ParentRelative);
	cpuFreqLabel->setBackgroundMode(X11ParentRelative);
//	setBackgroundOrigin(AncestorOrigin);
//	cpuTempLabel->setBackgroundOrigin(AncestorOrigin);
//	cpuFreqLabel->setBackgroundOrigin(AncestorOrigin);

	// add the labels to the layout
	layout->add(cpuTempLabel);
	layout->add(cpuFreqLabel);

	// create the menu
	menu = new KPopupMenu( this );
	menu->insertTitle( SmallIcon( "hwinfo" ), i18n("CPU Info") );

	thermalSources = new KSelectAction(i18n("&Thermal Sources"), KShortcut(), this, "ThermalSourcesKSelectAction");
	thermalSources->setItems(cpudata.getThermalSourceList());
	connect(thermalSources, SIGNAL(activated(int)), this, SLOT(setThermalSource(int)));
	connect(this, SIGNAL(thermalSourceChanged(int)), thermalSources, SLOT(setCurrentItem(int)));
	thermalSources->plug(menu);

	frequencySources = new KSelectAction(i18n("&Frequency Sources"), KShortcut(), this, "FrequencySourcesKSelectAction");
	frequencySources->setItems(cpudata.getFrequencySourceList());
	connect(frequencySources, SIGNAL(activated(int)), this, SLOT(setFrequencySource(int)));
	connect(this, SIGNAL(frequencySourceChanged(int)), frequencySources, SLOT(setCurrentItem(int)));
	frequencySources->plug(menu);

	if (cpufreqd.enabled()) {
		menu->insertItem(i18n("&Performance Profiles"), cpufreqd.menu());
	}
	
	KActionCollection* actionCollection = new KActionCollection( this, "actionCollection", kapp );

	KAction* action = KStdAction::preferences( this, SLOT( preferences() ), actionCollection );
	action->setText( i18n("&Preferences") );
	action->plug( menu );

	action = KStdAction::aboutApp( this, SLOT( about() ), actionCollection );
	action->setText( i18n("&About CPU Info") );
	action->plug( menu );

	//menu->insertSeparator();
	//KStdAction::quit( kapp, SLOT( quit() ), actionCollection )->plug( menu );

	// connect the timer
	connect( refreshTimer, SIGNAL(timeout()), this, SLOT(updateValues()) );

	//load prefs from the config
	loadPreferences();

	// fetch the latest temperature & frequency
	updateValues();

	// start the timer
	refreshTimer->start( sampleRate );
}

CPUInfo::~CPUInfo() {
	refreshTimer->stop();
	delete refreshTimer;
	//super class deletes the widgets
}

void CPUInfo::updateValues() {
	// fetch the current cpu temperature
	QString value;
	if(thermalItem >= 0)
		value = cpudata.getThermalSources()[thermalItem]->getValue();
	else
		value = "n/a";
	//set Label Tex if necessary
	if(cpuTempLabel->text() != value)
		cpuTempLabel->setText( value );

	// fetch the current cpu frequency
	if(frequencyItem >= 0)
		value = cpudata.getFrequencySources()[frequencyItem]->getValue();
	else
		value = "n/a";
	//set Label Tex if necessary
	if(cpuFreqLabel->text() != value)
		cpuFreqLabel->setText( value );

	updateWidths();

	// make the kicker relayout the panel and requery widthForHeight() or heightForWidth()
	//emit updateLayout();
}

void CPUInfo::about() {
	if(!aboutDialog){
		KAboutData* aboutData = new KAboutData("libcpuinfo", 
			I18N_NOOP("CPUInfo"),
			"0.4.3", 
			I18N_NOOP("Temperature/Frequency Applet.\n\n"
				"This applet shows the current CPU temperature and frequency.\n"
				"Make sure you have enabled a supported kernel module.\n"),
			KAboutData::License_GPL, "(c) 2004-2006, Ken Werner");
		aboutData->addAuthor("Ken Werner", 0, "ken.werner@web.de");
		aboutData->addAuthor("Valentine Sinitsyn - cpufreqd control module", 0, "e_val@inbox.ru");
		aboutData->addAuthor("Flavio Castelli - iBook G4 thermal source", 0, "micron@madlab.it");
		aboutData->addAuthor("Jocke Andersson - aligmnent improvement", 0, "ajocke@gmail.com");
		//aboutData->addCredit("blabla");
		aboutDialog = new KAboutApplication(aboutData, this, "aboutApp", false);
	}
	aboutDialog->show();
}

void CPUInfo::help() {
	KMessageBox::information(0, i18n("This is a help box"));
}

void CPUInfo::preferences() {
	//KMessageBox::information(0, i18n("This is a preferences box"));
	if(!prefsDlg){
		//create the prefs dialog
		prefsDlg = new KDialogBase( this, i18n("prefsDlg"), false, "Configuration", KDialogBase::Ok|KDialogBase::Apply|KDialogBase::Cancel, KDialogBase::Ok, false );
		//create the prefs ui
		prefs = new Prefs( prefsDlg, "prefsui" ); //the uic generated class

		//setup the dialog
		prefsDlg->setMainWidget(prefs); //embed the uic generated class into the kde dialog
		connect(prefsDlg, SIGNAL(applyClicked()), this, SLOT(savePreferences()));
		connect(prefsDlg, SIGNAL(okClicked()), this, SLOT(savePreferences()));
		connect(prefs->temperatureComboBox, SIGNAL(activated(int)), this, SLOT(setTempDescription(int)));
		connect(prefs->frequencyComboBox, SIGNAL(activated(int)), this, SLOT(setFreqDescription(int)));

		//alignment boxes
		const char* alignments[] = { "Left", "Center", "Right", 0 };
		prefs->tempAlignBox->clear();
		prefs->tempAlignBox->insertStrList(alignments);
		prefs->freqAlignBox->clear();
		prefs->freqAlignBox->insertStrList(alignments);

		//set the current appearance prefs
		prefs->tempColorButton->setColor(cpuTempLabel->foregroundColor());
		prefs->tempFontRequester->setFont(cpuTempLabel->font());
		if(cpuTempLabel->alignment() == Qt::AlignLeft)
			prefs->tempAlignBox->setCurrentItem(0);
		if(cpuTempLabel->alignment() == Qt::AlignRight)
			prefs->tempAlignBox->setCurrentItem(2);
		else
			prefs->tempAlignBox->setCurrentItem(1);

		prefs->freqColorButton->setColor(cpuFreqLabel->foregroundColor());
		prefs->freqFontRequester->setFont(cpuFreqLabel->font());
		if(cpuFreqLabel->alignment() == Qt::AlignLeft)
			prefs->freqAlignBox->setCurrentItem(0);
		if(cpuFreqLabel->alignment() == Qt::AlignRight)
			prefs->freqAlignBox->setCurrentItem(2);
		else
			prefs->freqAlignBox->setCurrentItem(1);

		prefs->spinBoxSampleRate->setValue(sampleRate);
	}

	//update thermal sources
	prefs->temperatureComboBox->clear(); // delete all entries
	thermalSources->clear(); // delete all entries
	cpudata.findThermalSources();
	prefs->temperatureComboBox->insertStringList(cpudata.getThermalSourceList());
	thermalSources->setItems(cpudata.getThermalSourceList());

	//update frequency sources
	prefs->frequencyComboBox->clear(); // delete all entries
	frequencySources->clear(); // delete all entries
	cpudata.findFrequencySources();
	prefs->frequencyComboBox->insertStringList(cpudata.getFrequencySourceList());
	frequencySources->setItems(cpudata.getFrequencySourceList());

	// set thermal source
	if(thermalItem >= cpudata.getThermalSources().size()) // check if the sensor is still available
		setThermalSource(cpudata.getThermalSources().size() - 1);
	if(thermalItem >= 0){
		prefs->temperatureComboBox->setCurrentItem(thermalItem);
		setTempDescription(thermalItem);
		thermalSources->setCurrentItem(thermalItem);
	}	

	// set frequency source
	if(frequencyItem >= cpudata.getFrequencySources().size()) // check if the sensor is still available
		setFrequencySource(cpudata.getFrequencySources().size() - 1);
	if(frequencyItem >= 0){
		prefs->frequencyComboBox->setCurrentItem(frequencyItem);
		setFreqDescription(frequencyItem);
		frequencySources->setCurrentItem(frequencyItem);
	}

	//show the dialog
	prefsDlg->show();
}

void CPUInfo::savePreferences(){
	//temperature color
	QColor tempColor = prefs->tempColorButton->color();
	if(cpuTempLabel->foregroundColor() != tempColor ){
		cpuTempLabel->setPaletteForegroundColor(tempColor);
		ksConfig->writeEntry( "tempColor", tempColor );
	}
	//temperature font
	QFont tempFont = prefs->tempFontRequester->font();
	if(cpuTempLabel->font() != tempFont){
		cpuTempLabel->setFont(tempFont);		
		ksConfig->writeEntry( "tempFont", tempFont );
	}
	//temperature alignment
	int tempAlignId = prefs->tempAlignBox->currentItem();
	Qt::AlignmentFlags tempAlign = Qt::AlignHCenter;
	if(tempAlignId == 0){
		tempAlign = Qt::AlignLeft;
	}else if(tempAlignId == 2){
		tempAlign = Qt::AlignRight;
	}
	if(cpuTempLabel->alignment() != tempAlign){
		cpuTempLabel->setAlignment(tempAlign);		
		ksConfig->writeEntry( "tempAlign", tempAlign );
	}
	//frequency color
	QColor freqColor = prefs->freqColorButton->color();
	if(cpuFreqLabel->foregroundColor() != freqColor ){
		cpuFreqLabel->setPaletteForegroundColor(freqColor);
		ksConfig->writeEntry( "freqColor", freqColor );
	}
	//frequency font
	QFont freqFont = prefs->freqFontRequester->font();
	if(cpuFreqLabel->font() != freqFont){
		cpuFreqLabel->setFont(freqFont);
		ksConfig->writeEntry( "freqFont", freqFont );
	}
	//frequency alignment
	int freqAlignId = prefs->freqAlignBox->currentItem();
	Qt::AlignmentFlags freqAlign = Qt::AlignHCenter;
	if(freqAlignId == 0){
		freqAlign = Qt::AlignLeft;
	}else if(freqAlignId == 2){
		freqAlign = Qt::AlignRight;
	}
	if(cpuFreqLabel->alignment() != freqAlign){
		cpuFreqLabel->setAlignment(freqAlign);		
		ksConfig->writeEntry( "freqAlign", freqAlign );
	}
	//sample rate
	if(sampleRate != prefs->spinBoxSampleRate->value()){
		sampleRate = prefs->spinBoxSampleRate->value();
		refreshTimer->changeInterval(sampleRate);
		ksConfig->writeEntry( "sampleRate", sampleRate );
	}
	
	if(prefs->temperatureComboBox->currentItem() < cpudata.getThermalSources().size()){
		setThermalSource(prefs->temperatureComboBox->currentItem());
		ksConfig->writeEntry( "thermalItem", thermalItem );
	}
	if(prefs->frequencyComboBox->currentItem() < cpudata.getFrequencySources().size()){	
		setFrequencySource(prefs->frequencyComboBox->currentItem());
		ksConfig->writeEntry( "frequencyItem", frequencyItem );
	}

	ksConfig->sync();

	updateWidths();

	// reset cached witdh for height
	cachedWFH = 0;
}

void CPUInfo::loadPreferences(){

	int tmp = ksConfig->readNumEntry( "sampleRate" );
	if(tmp != 0) //check whether the entry is set or not
		sampleRate = tmp;

	cpuTempLabel->setFont(ksConfig->readFontEntry("tempFont"));
	cpuTempLabel->setPaletteForegroundColor(ksConfig->readColorEntry("tempColor"));
	cpuTempLabel->setAlignment(ksConfig->readNumEntry("tempAlign", 4));

	cpuFreqLabel->setFont(ksConfig->readFontEntry("freqFont"));
	cpuFreqLabel->setPaletteForegroundColor(ksConfig->readColorEntry("freqColor"));
	cpuFreqLabel->setAlignment(ksConfig->readNumEntry("freqAlign", 4));

	if(ksConfig->readNumEntry("thermalItem") < cpudata.getThermalSources().size())
		setThermalSource(ksConfig->readNumEntry("thermalItem"));

	if(ksConfig->readNumEntry("frequencyItem") <  cpudata.getFrequencySources().size())
		setFrequencySource(ksConfig->readNumEntry("frequencyItem"));
}

int CPUInfo::widthForHeight(int height) const {
	//kdDebug() << "widthForHeight: " << height << endl;
	layout->setOrientation(Qt::Horizontal);
	cpuFreqLabel->repaint(); // the layout has changed
	cpuTempLabel->repaint(); // the layout has changed
	if(cachedHeight != height){
		cachedHeight = height;
		cachedWFH = 0;
	}
 	cachedWFH = QMAX(layout->widthForHeight(height), cachedWFH);
	return cachedWFH;
	//return sizeHint().width();
}

int CPUInfo::heightForWidth(int width) const {
	//kdDebug() << "heightForWidth: " << width << endl;
	layout->setOrientation(Qt::Vertical);
	cpuFreqLabel->repaint(); // the layout has changed
	cpuTempLabel->repaint(); // the layout has changed
	return layout->heightForWidth(width);
	//return sizeHint().height();
}

void CPUInfo::mousePressEvent( QMouseEvent* e ) {
    if( e->button() == QMouseEvent::RightButton )
        menu->popup( e->globalPos() );
}

void CPUInfo::setTempDescription(int index) {
	if(index >= 0)
		prefs->temperatureTextEdit->setText(cpudata.getThermalSources()[index]->getDescription());
}
void CPUInfo::setFreqDescription(int index) {
	if(index >= 0)
		prefs->frequencyTextEdit->setText(cpudata.getFrequencySources()[index]->getDescription());
}

void CPUInfo::setThermalSource(int index){
	if(index >= 0 && index < cpudata.getThermalSources().size() && index != thermalItem){
		thermalItem = index;
		emit thermalSourceChanged(thermalItem);
	}
}
void CPUInfo::setFrequencySource(int index){
	if(index >= 0 && index < cpudata.getFrequencySources().size() && index != frequencyItem){
		frequencyItem = index;
		emit frequencySourceChanged(frequencyItem);
	}
}

void CPUInfo::maybeTip( const QPoint& pos ){
	if(!rect().contains(pos))
		return;
	QString text = "<b>Thermal Sources:</b><br><table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">";
	for(uint i = 0; i < cpudata.getThermalSources().size(); i++){
		if(i == thermalItem)
			text.append("<tr><td><i>" + 
				cpudata.getThermalSources()[i]->getName() + "</i></td><td><i>" + 
				cpudata.getThermalSources()[i]->getValue()) + "</i></td></tr>";
		else
			text.append("<tr><td>" + 
				cpudata.getThermalSources()[i]->getName() + "</td><td>" + 
				cpudata.getThermalSources()[i]->getValue()) + "</td></tr>";
	}
	text.append("</table><br><b>Frequency Sources:</b><br><table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">");
	for(uint i = 0; i < cpudata.getFrequencySources().size(); i++){
		if(i == frequencyItem)
			text.append("<tr><td><i>" + 
				cpudata.getFrequencySources()[i]->getName() + "</i></td><td><i>" + 
				cpudata.getFrequencySources()[i]->getValue()) + "</i></td></tr>";
		else
			text.append("<tr><td>" + 
				cpudata.getFrequencySources()[i]->getName() + "</td><td>" + 
				cpudata.getFrequencySources()[i]->getValue()) + "</td></tr>";
	}
	text.append("</table>");
    tip(rect(), text);
}

void CPUInfo::updateWidths(){
	#define max(a,b) (((a)>(b))?(a):(b))
	int w = max(cpuFreqLabel->sizeHint().width(), cpuTempLabel->sizeHint().width());
	#undef max
	cpuTempLabel->setFixedWidth(w);
	cpuFreqLabel->setFixedWidth(w);
}
