/***************************************************************************
 *   Copyright (C) 2005 by Ken Werner                                      *
 *   ken.werner@web.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CPUINFO_H
#define CPUINFO_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kpanelapplet.h>
#include "cpudata.h"
#include "cpufreqd/cpufreqd.h"

// Qt
#include <qtooltip.h>
#include <qfont.h>
#include <qcolor.h>


// Forward Declarations
class QLabel;
class FlowLayout;
class KAboutApplication;
class KPopupMenu;
class KSelectAction;
class QMouseEvent;
class KDialogBase;
class Prefs;

/**
@author Ken Werner
*/
class CPUInfo : public KPanelApplet, QToolTip {
	Q_OBJECT //macro which activates signals and slots (moc)

public:
	/**
	 * Construct a @ref KPanelApplet just like any other widget.
	 *
	 * @param configFile The configFile handed over in the factory function.
	 * @param Type The applet @ref type().
	 * @param actions Standard RMB menu actions supported by the applet (see @ref action() ).
	 * @param parent The pointer to the parent widget handed over in the factory function.
	 * @param name A Qt object name for your applet.
	 **/
	CPUInfo(const QString& configFile, Type t = Normal, int actions = 0,
	           QWidget *parent = 0, const char *name = 0);
	/** destructor */
	virtual ~CPUInfo();

	/**
	 * Retrieve a suggested width for a given height.
	 *
	 * Every applet should reimplement this function.
	 *
	 * Depending on the panel orientation the height (horizontal panel) or the
	 * width (vertical panel) of the applets is fixed.
	 * The exact values of the fixed size component depend on the panel size.
	 *
	 * On a horizontal panel the applet height is fixed, the panel will
	 * call @ref widthForHeight(int height) with @p height
	 * equal to 'the fixed applet height'
	 * when laying out the applets.
	 *
	 * The applet can now choose the other size component (width)
	 * based on the given height.
	 *
	 * The width you return is granted.
	 **/
	virtual int widthForHeight(int height) const;

	/**
	 * @return A suggested height for a given width.
	 *
	 * Every applet should reimplement this function.
	 *
	 * Depending on the panel orientation the height (horizontal panel) or the
	 * width (vertical panel) of the applets is fixed.
	 * The exact values of the fixed size component depend on the panel size.
	 *
	 * On a vertical panel the applet width is fixed, the panel will
	 * call @ref heightForWidth(int width) with @p width
	 * equal to 'the fixed applet width'
	 * when laying out the applets.
	 *
	 * The applet can now choose the other size component (height)
	 * based on the given width.
	 *
	 * The height you return is granted.
	 **/
	virtual int heightForWidth(int width) const;
	
	/**
	 * Loads the preferences
	 **/
	void loadPreferences();

public slots:
	/**
	 * Saves the preferences
	 **/
	void savePreferences();

	/**
	 * Is called when the user selects "About" from the applets RMB menu.
	 * Reimplement this function to launch a about dialog.
	 *
	 * Note that this is called only when your applet supports the About action.
	 * See @ref Action and @ref KPanelApplet().
	 **/
	virtual void about();

	/**
	 * Is called when the user selects "Help" from the applets RMB menu.
	 * Reimplement this function to launch a manual or help page.
	 *
	 * Note that this is called only when your applet supports the Help action.
	 * See @ref Action and @ref KPanelApplet().
	 **/
	virtual void help();

	/**
	 * Is called when the user selects "Preferences" from the applets RMB menu.
	 * Reimplement this function to launch a preferences dialog or kcontrol module.
	 *
	 * Note that this is called only when your applet supports the preferences action.
	 * See @ref Action and @ref KPanelApplet().
	 **/
	virtual void preferences();

protected:
	void mousePressEvent( QMouseEvent* e );
	void maybeTip( const QPoint& pos);
private:
	void updateWidths();
	KConfig* ksConfig;
	FlowLayout* layout;
	mutable int cachedWFH;
	mutable int cachedHeight;
	QLabel* cpuTempLabel;
	QLabel* cpuFreqLabel;
	KAboutApplication* aboutDialog;
	QTimer* refreshTimer;
	KPopupMenu* menu;
	KSelectAction* thermalSources;
	KSelectAction* frequencySources;
	KDialogBase* prefsDlg;
	Prefs* prefs;

	// preferences
	int sampleRate;
	int thermalItem;
	int frequencyItem;

	CPUData cpudata;
	CPUFreqd cpufreqd;

private slots:
	void updateValues();
	void setThermalSource(int index);
	void setFrequencySource(int index);
	void setTempDescription(int index);
	void setFreqDescription(int index);

signals:
	void thermalSourceChanged(int index);
	void frequencySourceChanged(int index);
};

#endif
